import React, { createContext, forwardRef, useContext, useRef, useMemo, useState, useCallback } from 'react';
import { VariableSizeGrid } from 'react-window';
export { areEqual } from 'react-window';

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

var UpdateItemAction;

(function (UpdateItemAction) {
  UpdateItemAction[UpdateItemAction["MOVE"] = 0] = "MOVE";
  UpdateItemAction[UpdateItemAction["RESIZE"] = 1] = "RESIZE";
})(UpdateItemAction || (UpdateItemAction = {}));

var TimelineContext = /*#__PURE__*/createContext({
  GroupRenderer: function GroupRenderer() {
    return null;
  },
  ItemRenderer: function ItemRenderer() {
    return null;
  },
  BodyRenderer: function BodyRenderer() {
    return null;
  },
  SidebarHeaderRenderer: function SidebarHeaderRenderer() {
    return null;
  },
  SidebarRenderer: function SidebarRenderer() {
    return null;
  },
  TimebarHeaderRenderer: function TimebarHeaderRenderer() {
    return null;
  },
  TimebarIntervalRenderer: function TimebarIntervalRenderer() {
    return null;
  },
  columnCount: 0,
  columnWidth: function columnWidth() {
    return 0;
  },
  endTime: 0,
  getUpdatedItem: function getUpdatedItem() {
    return null;
  },
  createItemAtCursor: function createItemAtCursor() {
    return null;
  },
  groups: [],
  height: 0,
  intervalDuration: 0,
  intervalWidth: 0,
  intervals: [],
  itemData: {},
  itemHeight: 0,
  itemMap: /*#__PURE__*/new Map(),
  minItemWidth: 0,
  outerRef: {
    current: null
  },
  overscanColumnStartIndex: 0,
  overscanColumnStopIndex: 0,
  overscanRowStartIndex: 0,
  overscanRowStopIndex: 0,
  removeItem: function removeItem() {
    return undefined;
  },
  rowCount: 0,
  rowHeight: function rowHeight() {
    return 0;
  },
  rowMap: /*#__PURE__*/new Map(),
  setStickyItemIds: function setStickyItemIds() {
    return undefined;
  },
  sidebarWidth: 0,
  snapDuration: 0,
  startTime: 0,
  stickyItemIds: [],
  timebarHeaderHeight: 0,
  timebarHeight: 0,
  timebarIntervalHeight: 0,
  upsertItem: function upsertItem() {
    return undefined;
  },
  visibleColumnStartIndex: 0,
  visibleColumnStopIndex: 0,
  visibleRowStartIndex: 0,
  visibleRowStopIndex: 0,
  width: 0
});
TimelineContext.displayName = 'TimelineContext';

function rangeInclusive(start, stop) {
  var result = [];

  for (var i = start; i <= stop; i += 1) {
    result.push(i);
  }

  return result;
}

function getIntervals(startDate, endDate, interval) {
  var startTime = startDate.getTime();
  var endTime = endDate.getTime(); // Throw an exception if start date is after end date or if any date is `Invalid Date`

  if (!(startTime <= endTime)) {
    throw new Error('Invalid start and end time');
  }

  var dates = [];
  var currentTime = startTime;

  while (currentTime < endTime) {
    dates.push(currentTime);
    currentTime += interval;
  }

  return dates;
}
function snapTime(time, snapDuration) {
  if (snapDuration <= 0) {
    return time;
  }

  return Math.round(time / snapDuration) * snapDuration;
}
function getPositionAtTime(time, startTime, intervalDuration, intervalWidth) {
  var numberOfIntervals = (time - startTime) / intervalDuration;
  var left = numberOfIntervals * intervalWidth;
  return left;
}
function getTimeAtPosition(left, startTime, intervalDuration, intervalWidth, snapDuration) {
  var numberOfIntervals = left / intervalWidth;
  return snapTime(numberOfIntervals * intervalDuration + startTime, snapDuration);
}

var innerElementType = /*#__PURE__*/forwardRef(function TimelineInner(_ref, ref) {
  var _visibleRows$, _visibleRows$2;

  var style = _ref.style;

  var _useContext = useContext(TimelineContext),
      BodyRenderer = _useContext.BodyRenderer,
      ColumnRenderer = _useContext.ColumnRenderer,
      GroupRenderer = _useContext.GroupRenderer,
      ItemRenderer = _useContext.ItemRenderer,
      RowRenderer = _useContext.RowRenderer,
      SidebarHeaderRenderer = _useContext.SidebarHeaderRenderer,
      SidebarRenderer = _useContext.SidebarRenderer,
      TimebarHeaderRenderer = _useContext.TimebarHeaderRenderer,
      TimebarIntervalRenderer = _useContext.TimebarIntervalRenderer,
      childrenFromProps = _useContext.children,
      intervalDuration = _useContext.intervalDuration,
      intervalWidth = _useContext.intervalWidth,
      itemHeight = _useContext.itemHeight,
      itemMap = _useContext.itemMap,
      minItemWidth = _useContext.minItemWidth,
      overscanColumnStartIndex = _useContext.overscanColumnStartIndex,
      overscanColumnStopIndex = _useContext.overscanColumnStopIndex,
      overscanRowStartIndex = _useContext.overscanRowStartIndex,
      overscanRowStopIndex = _useContext.overscanRowStopIndex,
      rowMap = _useContext.rowMap,
      sidebarWidth = _useContext.sidebarWidth,
      startTime = _useContext.startTime,
      stickyItemIds = _useContext.stickyItemIds,
      timebarHeight = _useContext.timebarHeight,
      timebarHeaderHeight = _useContext.timebarHeaderHeight,
      timebarIntervalHeight = _useContext.timebarIntervalHeight;

  var stickyItems = stickyItemIds.map(function (id) {
    return itemMap.get(id);
  });
  var stickyRows = stickyItems.map(function (item) {
    return item.row;
  });
  var visibleRows = rowMap.size > 0 ? rangeInclusive(overscanRowStartIndex, overscanRowStopIndex).map(function (rowIndex) {
    return rowMap.get(rowIndex);
  }) : [];
  var rows = Array.from(new Set([].concat(stickyRows, visibleRows))).sort(function (a, b) {
    return a.index - b.index;
  });
  var columns = rangeInclusive(overscanColumnStartIndex, overscanColumnStopIndex);
  return React.createElement(React.Fragment, null, childrenFromProps, ColumnRenderer && React.createElement("div", {
    style: {
      gridArea: '3 / 1 / 4 / 3',
      position: 'sticky',
      top: timebarHeight
    }
  }, columns.map(function (column) {
    return React.createElement(ColumnRenderer, {
      key: "column:" + column,
      isOdd: column % 2 === 1,
      isEven: column % 2 === 0,
      time: startTime + intervalDuration * column,
      style: {
        position: 'absolute',
        boxSizing: 'border-box',
        top: 0,
        left: intervalWidth * column + sidebarWidth,
        width: intervalWidth,
        bottom: 0,
        userSelect: 'none',
        pointerEvents: 'none'
      }
    });
  })), RowRenderer && React.createElement("div", {
    style: {
      gridArea: '3 / 2 / 4 / 4',
      position: 'sticky',
      left: 0,
      marginTop: ((_visibleRows$ = visibleRows[0]) == null ? void 0 : _visibleRows$.top) || 0,
      minWidth: style.width
    }
  }, visibleRows.flatMap(function (row) {
    return React.createElement(RowRenderer, {
      key: "row:" + row.index,
      rowIndex: row.index,
      group: row.group,
      isOdd: row.index % 2 === 1,
      isEven: row.index % 2 === 0,
      style: {
        position: 'sticky',
        display: 'block',
        boxSizing: 'border-box',
        height: row.height,
        left: 0
      }
    });
  })), React.createElement(BodyRenderer, {
    ref: ref,
    style: {
      whiteSpace: 'nowrap',
      gridArea: '3 / 2 / 3 / 4',
      position: 'relative',
      minHeight: style.height,
      minWidth: style.width
    }
  }, rows.map(function (row) {
    var rowStickyItems = stickyItems.filter(function (item) {
      return item.groupId === row.group.id;
    });
    var isStickyRow = row.index < overscanRowStartIndex || row.index > overscanRowStopIndex;

    function renderItem(item) {
      var left = getPositionAtTime(item.start, startTime, intervalDuration, intervalWidth);
      var width = Math.max(getPositionAtTime(item.end, startTime, intervalDuration, intervalWidth) - left, minItemWidth);
      var top = row.top + row.getItemTopOffset(item, itemHeight);
      var isStickyItem = rowStickyItems.some(function (_ref2) {
        var id = _ref2.id;
        return id === item.id;
      });
      return React.createElement(ItemRenderer, {
        key: item.id,
        item: item,
        style: {
          position: 'absolute',
          boxSizing: 'border-box',
          left: left,
          top: top,
          width: width,
          height: itemHeight,
          zIndex: isStickyItem ? 1 : 0,
          userSelect: 'none'
        }
      });
    } // Only render the items if this is a sticky row


    if (isStickyRow) {
      return rowStickyItems.map(function (item) {
        return renderItem(item);
      });
    }

    return Array.from(new Set([].concat(rowStickyItems, row.getItemsByIntervalRange(overscanColumnStartIndex, overscanColumnStopIndex)))).map(function (item) {
      return renderItem(item);
    });
  })), React.createElement(SidebarRenderer, {
    style: {
      gridArea: '1 / 1 / 4 / 2',
      position: 'sticky',
      left: 0,
      minHeight: style.height,
      paddingTop: (((_visibleRows$2 = visibleRows[0]) == null ? void 0 : _visibleRows$2.top) || 0) + timebarHeight,
      boxSizing: 'border-box'
    }
  }, visibleRows.flatMap(function (row) {
    return [sidebarWidth > 0 && GroupRenderer && React.createElement(GroupRenderer, {
      key: "group:" + row.index,
      rowIndex: row.index,
      group: row.group,
      isOdd: row.index % 2 === 1,
      isEven: row.index % 2 === 0,
      style: {
        position: 'sticky',
        display: 'block',
        boxSizing: 'border-box',
        width: sidebarWidth,
        height: row.height,
        left: 0,
        userSelect: 'none'
      }
    })];
  })), React.createElement("div", {
    style: {
      whiteSpace: 'nowrap',
      position: 'sticky',
      boxSizing: 'border-box',
      top: timebarHeaderHeight,
      gridArea: '2 / 1 / 3 / 3',
      overflow: 'visible',
      marginLeft: columns[0] * intervalWidth + sidebarWidth
    }
  }, columns.flatMap(function (column) {
    return [TimebarIntervalRenderer && React.createElement(TimebarIntervalRenderer, {
      key: "timebar_interval:" + column,
      isOdd: column % 2 === 1,
      isEven: column % 2 === 0,
      time: startTime + intervalDuration * column,
      style: {
        position: 'sticky',
        display: 'inline-flex',
        boxSizing: 'border-box',
        width: intervalWidth,
        top: timebarHeaderHeight,
        height: timebarIntervalHeight,
        verticalAlign: 'top',
        userSelect: 'none'
      }
    })];
  })), React.createElement(SidebarHeaderRenderer, {
    key: "sidebar:header",
    style: {
      position: 'sticky',
      boxSizing: 'border-box',
      top: timebarHeaderHeight,
      left: 0,
      gridArea: '2 / 1 / 3 / 2'
    }
  }), timebarHeaderHeight > 0 && TimebarHeaderRenderer && React.createElement(TimebarHeaderRenderer, {
    key: "timebar:head",
    style: {
      position: 'sticky',
      top: 0,
      left: 0,
      zIndex: 3,
      gridArea: '1 / 1 / 2 / 3'
    }
  }));
});

var Row = /*#__PURE__*/function () {
  function Row(_ref) {
    var group = _ref.group,
        index = _ref.index,
        intervals = _ref.intervals,
        itemHeight = _ref.itemHeight,
        itemMap = _ref.itemMap,
        rowMap = _ref.rowMap,
        timebarHeight = _ref.timebarHeight,
        minGroupHeight = _ref.minGroupHeight,
        groupTopPadding = _ref.groupTopPadding,
        groupBottomPadding = _ref.groupBottomPadding;
    this.itemIds = new Set();
    this.offsets = {};
    this.group = group;
    this.index = index;
    this.intervals = intervals;
    this.itemHeight = itemHeight;
    this.itemMap = itemMap;
    this.rowMap = rowMap;
    this.timebarHeight = timebarHeight;
    this.minGroupHeight = minGroupHeight;
    this.groupTopPadding = groupTopPadding;
    this.groupBottomPadding = groupBottomPadding;
  }

  var _proto = Row.prototype;

  _proto.recalculate = function recalculate() {
    this.offsets = {};
    var sortedItems = this.items.sort(function (a, b) {
      if (a.start === b.start) {
        return a.end - b.end;
      }

      return a.start - b.start;
    });
    var rowOffset = 0; // Loop until everything has been calculated

    while (sortedItems.length > 0) {
      var lastEnd = null; // Find all items that do not overlap

      for (var index = 0; index < sortedItems.length; index++) {
        var item = sortedItems[index]; // This item does not overlap with the last

        if (lastEnd === null || item.start > lastEnd) {
          // Therefore it can have the same rowOffset
          this.offsets[item.id] = rowOffset; // Remove this item as it has been processed

          sortedItems.splice(index, 1); // Keep the index in place, as we modified the array

          index--; // The next item needs to check against this one

          lastEnd = item.end;
        }
      } // All the remaining items overlapped with something, so increase the rowOffset
      // We will also reset the lastEnd when we start looping again


      rowOffset++;
    }

    var height = this.groupTopPadding + Math.max(rowOffset, 1) * this.itemHeight + this.groupBottomPadding;
    this._height = Math.max(this.minGroupHeight, height);
  };

  _proto.addItem = function addItem(item) {
    this.itemIds.add(item.id); // Force the height be be recalculated

    this._height = undefined;
  };

  _proto.removeItem = function removeItem(item) {
    this.itemIds["delete"](item.id); // Force the height be be recalculated

    this._height = undefined;
  };

  _proto.getItemTopOffset = function getItemTopOffset(item, itemHeight) {
    return this.offsets[item.id] * itemHeight + this.groupTopPadding;
  };

  _proto.getItemsByIntervalRange = function getItemsByIntervalRange(startIndex, endIndex) {
    var start = this.intervals[startIndex];
    var end = this.intervals[endIndex];
    return this.items.filter(function (item) {
      return item.end > start && item.start < end;
    });
  };

  _createClass(Row, [{
    key: "height",
    get: function get() {
      if (this._height === undefined) {
        this.recalculate();
      }

      return this._height;
    }
  }, {
    key: "top",
    get: function get() {
      if (this._height === undefined) {
        this.recalculate();
      }

      var previousRow = this.rowMap.get(this.index - 1);

      if (!previousRow) {
        return 0; //this.timebarHeight;
      }

      return previousRow.top + previousRow.height;
    }
  }, {
    key: "bottom",
    get: function get() {
      return this.height + this.top;
    }
  }, {
    key: "items",
    get: function get() {
      var _this = this;

      return Array.from(this.itemIds).map(function (id) {
        return _this.itemMap.get(id);
      });
    }
  }]);

  return Row;
}();

function getTimelineData(_ref) {
  var groups = _ref.groups,
      intervals = _ref.intervals,
      itemHeight = _ref.itemHeight,
      items = _ref.items,
      snapDuration = _ref.snapDuration,
      timebarHeight = _ref.timebarHeight,
      minGroupHeight = _ref.minGroupHeight,
      groupTopPadding = _ref.groupTopPadding,
      groupBottomPadding = _ref.groupBottomPadding;
  var itemMap = new Map();
  var rowMap = new Map();
  var groupMap = {};

  for (var _iterator = _createForOfIteratorHelperLoose(groups.entries()), _step; !(_step = _iterator()).done;) {
    var _step$value = _step.value,
        index = _step$value[0],
        group = _step$value[1];
    var row = new Row({
      group: group,
      index: index,
      intervals: intervals,
      itemHeight: itemHeight,
      itemMap: itemMap,
      rowMap: rowMap,
      timebarHeight: timebarHeight,
      minGroupHeight: minGroupHeight,
      groupTopPadding: groupTopPadding,
      groupBottomPadding: groupBottomPadding
    });
    rowMap.set(row.index, row);
    groupMap[group.id] = row;
  }

  for (var _iterator2 = _createForOfIteratorHelperLoose(items), _step2; !(_step2 = _iterator2()).done;) {
    var item = _step2.value;
    var _row = groupMap[item.groupId];

    if (_row === undefined) {
      throw new Error('Item has mismatched group id');
    }

    var mappedItem = _extends({}, item, {
      row: _row,
      start: snapTime(item.start, snapDuration),
      end: snapTime(item.end, snapDuration)
    });

    _row.addItem(mappedItem);

    itemMap.set(item.id, mappedItem);
  }

  return [itemMap, rowMap];
}
function getUnmappedItem(_ref2) {
  var item = _objectWithoutPropertiesLoose(_ref2, ["row"]);

  return item;
}

function Timeline(props) {
  var initialEndTime = props.endTime,
      groups = props.groups,
      height = props.height,
      intervalDuration = props.intervalDuration,
      intervalWidth = props.intervalWidth,
      itemRenderer = props.itemRenderer,
      items = props.items,
      initialStartTime = props.startTime,
      width = props.width,
      _props$bodyRenderer = props.bodyRenderer,
      bodyRenderer = _props$bodyRenderer === void 0 ? forwardRef(function (props, ref) {
    return React.createElement("div", Object.assign({
      ref: ref
    }, props));
  }) : _props$bodyRenderer,
      children = props.children,
      itemData = props.itemData,
      _props$itemHeight = props.itemHeight,
      itemHeight = _props$itemHeight === void 0 ? 20 : _props$itemHeight,
      _props$groupBottomPad = props.groupBottomPadding,
      groupBottomPadding = _props$groupBottomPad === void 0 ? 0 : _props$groupBottomPad,
      groupRenderer = props.groupRenderer,
      _props$groupTopPaddin = props.groupTopPadding,
      groupTopPadding = _props$groupTopPaddin === void 0 ? 0 : _props$groupTopPaddin,
      timebarIntervalRenderer = props.timebarIntervalRenderer,
      columnRenderer = props.columnRenderer,
      rowRenderer = props.rowRenderer,
      _props$minItemWidth = props.minItemWidth,
      minItemWidth = _props$minItemWidth === void 0 ? 5 : _props$minItemWidth,
      _props$minGroupHeight = props.minGroupHeight,
      minGroupHeight = _props$minGroupHeight === void 0 ? 0 : _props$minGroupHeight,
      _props$onItemUpdate = props.onItemUpdate,
      onItemUpdate = _props$onItemUpdate === void 0 ? function () {
    return undefined;
  } : _props$onItemUpdate,
      _props$sidebarWidth = props.sidebarWidth,
      sidebarWidth = _props$sidebarWidth === void 0 ? 0 : _props$sidebarWidth,
      _props$snapDuration = props.snapDuration,
      snapDuration = _props$snapDuration === void 0 ? 1000 * 60 : _props$snapDuration,
      _props$timebarInterva = props.timebarIntervalHeight,
      timebarIntervalHeight = _props$timebarInterva === void 0 ? itemHeight : _props$timebarInterva,
      _props$timebarHeaderH = props.timebarHeaderHeight,
      timebarHeaderHeight = _props$timebarHeaderH === void 0 ? 0 : _props$timebarHeaderH,
      timebarHeaderRenderer = props.timebarHeaderRenderer,
      _props$sidebarHeaderR = props.sidebarHeaderRenderer,
      sidebarHeaderRenderer = _props$sidebarHeaderR === void 0 ? function (props) {
    return React.createElement("div", Object.assign({}, props));
  } : _props$sidebarHeaderR,
      _props$sidebarRendere = props.sidebarRenderer,
      sidebarRenderer = _props$sidebarRendere === void 0 ? function (props) {
    return React.createElement("div", Object.assign({}, props));
  } : _props$sidebarRendere,
      _props$initialScrollL = props.initialScrollLeft,
      initialScrollLeft = _props$initialScrollL === void 0 ? 0 : _props$initialScrollL,
      initialScrollTime = props.initialScrollTime,
      style = props.style,
      gridProps = _objectWithoutPropertiesLoose(props, ["endTime", "groups", "height", "intervalDuration", "intervalWidth", "itemRenderer", "items", "startTime", "width", "bodyRenderer", "children", "itemData", "itemHeight", "groupBottomPadding", "groupRenderer", "groupTopPadding", "timebarIntervalRenderer", "columnRenderer", "rowRenderer", "minItemWidth", "minGroupHeight", "onItemUpdate", "sidebarWidth", "snapDuration", "timebarIntervalHeight", "timebarHeaderHeight", "timebarHeaderRenderer", "sidebarHeaderRenderer", "sidebarRenderer", "initialScrollLeft", "initialScrollTime", "style"]);

  var gridRef = useRef(null);
  var outerRef = useRef(null);
  var startTime = snapTime(initialStartTime, snapDuration);
  var endTime = snapTime(initialEndTime, snapDuration);
  var timebarHeight = timebarIntervalHeight + timebarHeaderHeight;
  var intervals = useMemo(function () {
    return getIntervals(new Date(startTime), new Date(endTime), intervalDuration);
  }, [startTime, endTime, intervalDuration]);

  var _useMemo = useMemo(function () {
    var _gridRef$current;

    (_gridRef$current = gridRef.current) == null ? void 0 : _gridRef$current.resetAfterRowIndex(0, false);
    return getTimelineData({
      groups: groups,
      items: items,
      intervals: intervals,
      itemHeight: itemHeight,
      timebarHeight: timebarHeight,
      snapDuration: snapDuration,
      minGroupHeight: minGroupHeight,
      groupTopPadding: groupTopPadding,
      groupBottomPadding: groupBottomPadding
    });
  }, [groups, items, intervals, itemHeight, timebarHeight, snapDuration, minGroupHeight, groupTopPadding, groupBottomPadding]),
      itemMap = _useMemo[0],
      rowMap = _useMemo[1];

  var _useState = useState([]),
      stickyItemIds = _useState[0],
      setStickyItemIds = _useState[1];

  var handleSetStickyItemIds = useCallback(function (ids) {
    setStickyItemIds(ids.map(function (id) {
      var item = itemMap.get(id);

      if (!item) {
        throw new Error('Item is not mapped');
      }

      return id;
    }));
  }, [setStickyItemIds, itemMap]);

  var _useState2 = useState({
    overscanColumnStartIndex: 0,
    overscanColumnStopIndex: 0,
    overscanRowStartIndex: 0,
    overscanRowStopIndex: 0,
    visibleColumnStartIndex: 0,
    visibleColumnStopIndex: 0,
    visibleRowStartIndex: 0,
    visibleRowStopIndex: 0
  }),
      _useState2$ = _useState2[0],
      overscanColumnStartIndex = _useState2$.overscanColumnStartIndex,
      overscanColumnStopIndex = _useState2$.overscanColumnStopIndex,
      overscanRowStartIndex = _useState2$.overscanRowStartIndex,
      overscanRowStopIndex = _useState2$.overscanRowStopIndex,
      visibleColumnStartIndex = _useState2$.visibleColumnStartIndex,
      visibleColumnStopIndex = _useState2$.visibleColumnStopIndex,
      visibleRowStartIndex = _useState2$.visibleRowStartIndex,
      visibleRowStopIndex = _useState2$.visibleRowStopIndex,
      setVisibleArea = _useState2[1];

  var getUpdatedItem = useCallback(function (event, item, action) {
    var _outerRef$current, _outerRef$current2, _outerRef$current3;

    if (!event.currentTarget) {
      return null;
    }

    var targetBox = event.currentTarget.getBoundingClientRect();
    var outerElement = ((_outerRef$current = outerRef.current) == null ? void 0 : _outerRef$current.getBoundingClientRect()) || {
      left: 0,
      right: 0,
      top: 0,
      bottom: 0
    };
    var scrollLeft = ((_outerRef$current2 = outerRef.current) == null ? void 0 : _outerRef$current2.scrollLeft) || 0;
    var scrollTop = ((_outerRef$current3 = outerRef.current) == null ? void 0 : _outerRef$current3.scrollTop) || 0;
    var left = targetBox.left + scrollLeft - outerElement.left - sidebarWidth;
    var right = targetBox.right + scrollLeft - outerElement.left - sidebarWidth;
    var top = targetBox.top + scrollTop - outerElement.top - timebarHeight;

    var updatedValues = _extends({}, item, {
      start: Math.max(startTime, getTimeAtPosition(left, startTime, intervalDuration, intervalWidth, snapDuration)),
      end: Math.min(endTime, getTimeAtPosition(right, startTime, intervalDuration, intervalWidth, snapDuration))
    });

    if (action !== UpdateItemAction.RESIZE) {
      var row = Array.from(rowMap.values()).find(function (row) {
        return row.top <= top && row.top + row.height > top;
      });

      if (!row) {
        return null;
      }

      updatedValues.groupId = row.group.id;
    }

    return updatedValues;
  }, [rowMap, startTime, endTime, intervalDuration, intervalWidth, sidebarWidth, snapDuration, timebarHeight]);
  var removeItem = useCallback(function (_ref) {
    var _gridRef$current2;

    var id = _ref.id;
    var item = itemMap.get(id);

    if (!item) {
      throw new Error('Unable to find item');
    }

    item.row.removeItem(item);
    itemMap["delete"](item.id);
    onItemUpdate(item);
    (_gridRef$current2 = gridRef.current) == null ? void 0 : _gridRef$current2.resetAfterRowIndex(item.row.index);
  }, [itemMap, onItemUpdate]);
  var upsertItem = useCallback(function (item) {
    var _gridRef$current3;

    var row = Array.from(rowMap.values()).find(function (_ref2) {
      var group = _ref2.group;
      return group.id === item.groupId;
    });

    if (!row) {
      throw new Error("Unable to find group with id " + item.groupId);
    }

    var previous = itemMap.get(item.id);

    if (previous && item.groupId !== previous.groupId) {
      previous.row.removeItem(item);
    }

    var updatedItem = Object.assign(item, {
      end: snapTime(item.end, snapDuration),
      groupId: row.group.id,
      row: row,
      start: snapTime(item.start, snapDuration)
    });
    row.addItem(item);
    itemMap.set(item.id, updatedItem);
    onItemUpdate(item);
    (_gridRef$current3 = gridRef.current) == null ? void 0 : _gridRef$current3.resetAfterRowIndex(Math.min(row.index, (previous == null ? void 0 : previous.row.index) || row.index));
  }, [itemMap, rowMap, snapDuration, onItemUpdate]);
  var createItemAtCursor = useCallback(function (event, _ref3) {
    var _outerRef$current4, _outerRef$current5, _outerRef$current6;

    var id = _ref3.id,
        duration = _ref3.duration,
        defaultGroupId = _ref3.defaultGroupId,
        rest = _objectWithoutPropertiesLoose(_ref3, ["id", "duration", "defaultGroupId"]);

    if (!event.currentTarget) {
      return null;
    }

    var outerElement = ((_outerRef$current4 = outerRef.current) == null ? void 0 : _outerRef$current4.getBoundingClientRect()) || {
      left: 0,
      right: 0,
      top: 0,
      bottom: 0
    };
    var mouseLeft = event.clientX;
    var mouseTop = event.clientY;
    var scrollLeft = ((_outerRef$current5 = outerRef.current) == null ? void 0 : _outerRef$current5.scrollLeft) || 0;
    var scrollTop = ((_outerRef$current6 = outerRef.current) == null ? void 0 : _outerRef$current6.scrollTop) || 0;
    var left = mouseLeft + scrollLeft - outerElement.left - sidebarWidth;
    var top = mouseTop + scrollTop - outerElement.top - timebarHeight;
    var row = Array.from(rowMap.values()).find(function (row) {
      return row.top <= top && row.top + row.height >= top;
    });
    var start = Math.max(startTime, getTimeAtPosition(left, startTime, intervalDuration, intervalWidth, snapDuration));

    var item = _extends({
      id: id,
      start: start,
      end: snapTime(start + duration, snapDuration),
      groupId: (row == null ? void 0 : row.group.id) || defaultGroupId || ''
    }, rest);

    if (!item.groupId) {
      throw new Error('No group id provided');
    }

    upsertItem(item);
    return item;
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  [sidebarWidth, timebarHeight, rowMap, startTime, intervalDuration, intervalWidth, snapDuration, upsertItem]);
  var BodyRenderer = bodyRenderer;
  var ColumnRenderer = columnRenderer;
  var GroupRenderer = groupRenderer;
  var ItemRenderer = itemRenderer;
  var RowRenderer = rowRenderer;
  var SidebarHeaderRenderer = sidebarHeaderRenderer;
  var SidebarRenderer = sidebarRenderer;
  var TimebarHeaderRenderer = timebarHeaderRenderer;
  var TimebarIntervalRenderer = timebarIntervalRenderer;
  var rowCount = groups.length; // debugger;

  var columnCount = intervals.length;
  var columnWidth = useCallback(function () {
    return intervalWidth;
  }, [intervalWidth]);
  var rowHeight = useCallback(function (rowIndex) {
    var _rowMap$get;

    return ((_rowMap$get = rowMap.get(rowIndex)) == null ? void 0 : _rowMap$get.height) || minGroupHeight || itemHeight;
  }, [rowMap, minGroupHeight, itemHeight]);
  var initialScrollX = !initialScrollTime ? initialScrollLeft : Math.max(0, getPositionAtTime(initialScrollTime, startTime, intervalDuration, intervalWidth) - sidebarWidth);
  var outerElementStyle = useMemo(function () {
    return _extends({
      display: 'grid',
      gridTemplateRows: timebarHeaderHeight + "px " + timebarIntervalHeight + "px 1fr",
      gridTemplateColumns: sidebarWidth + "px calc(100% - " + sidebarWidth + "px) 1fr"
    }, style);
  }, [timebarHeaderHeight, timebarIntervalHeight, sidebarWidth, style]);
  return React.createElement(TimelineContext.Provider, {
    value: {
      BodyRenderer: BodyRenderer,
      ColumnRenderer: ColumnRenderer,
      GroupRenderer: GroupRenderer,
      RowRenderer: RowRenderer,
      SidebarHeaderRenderer: SidebarHeaderRenderer,
      SidebarRenderer: SidebarRenderer,
      TimebarHeaderRenderer: TimebarHeaderRenderer,
      TimebarIntervalRenderer: TimebarIntervalRenderer,
      children: children,
      columnCount: columnCount,
      columnWidth: columnWidth,
      endTime: endTime,
      getUpdatedItem: getUpdatedItem,
      groups: groups,
      createItemAtCursor: createItemAtCursor,
      height: height,
      intervalDuration: intervalDuration,
      intervalWidth: intervalWidth,
      intervals: intervals,
      itemData: itemData,
      itemMap: itemMap,
      itemHeight: itemHeight,
      ItemRenderer: ItemRenderer,
      minItemWidth: minItemWidth,
      outerRef: outerRef,
      overscanColumnStartIndex: overscanColumnStartIndex,
      overscanColumnStopIndex: overscanColumnStopIndex,
      overscanRowStartIndex: overscanRowStartIndex,
      overscanRowStopIndex: overscanRowStopIndex,
      removeItem: removeItem,
      rowCount: rowCount,
      rowHeight: rowHeight,
      rowMap: rowMap,
      setStickyItemIds: handleSetStickyItemIds,
      sidebarWidth: sidebarWidth,
      snapDuration: snapDuration,
      startTime: startTime,
      stickyItemIds: stickyItemIds,
      timebarHeaderHeight: timebarHeaderHeight,
      timebarHeight: timebarHeight,
      timebarIntervalHeight: timebarIntervalHeight,
      upsertItem: upsertItem,
      visibleColumnStartIndex: visibleColumnStartIndex,
      visibleColumnStopIndex: visibleColumnStopIndex,
      visibleRowStartIndex: visibleRowStartIndex,
      visibleRowStopIndex: visibleRowStopIndex,
      width: width
    }
  }, React.createElement(VariableSizeGrid, Object.assign({}, gridProps, {
    ref: gridRef,
    columnCount: columnCount,
    columnWidth: columnWidth,
    estimatedColumnWidth: intervalWidth,
    height: height,
    initialScrollLeft: initialScrollX,
    innerElementType: innerElementType,
    onItemsRendered: setVisibleArea,
    outerRef: outerRef,
    rowCount: Math.max(rowCount, 1),
    rowHeight: rowHeight,
    style: outerElementStyle,
    width: width
  }), noopRenderer));
}

var noopRenderer = function noopRenderer() {
  return null;
};

export default Timeline;
export { TimelineContext, UpdateItemAction, getTimelineData, getUnmappedItem };
//# sourceMappingURL=react-window-timeline.esm.js.map
